{ RunOnce component version 1.0 copyright (c) 2000 Michal Cihar

LICENSE:
  This component may be used without any charge in non-commercial
applications. For commercial applications you must contact author
and recieve premission to use this. Complete license text can be
found in file LICENSE.TXT which should you get with this component.

CONTACTING AUTHOR:
  Email: cihar@email.cz
  WWW: http://cicom.tsx.org
  ICQ: 33471395

DESCRIPTION:
  Include component RunOnce in your project to ensure that only one
copy of your application is running. RunOnce also enables activating
another instance of application in any conditions (the only exception
is that you change IDString property and rerun CreateID).

PROPERTIES:
  Published; read/write:
    Terminate - if true application is terminated when another copy
                found
    Activate - if true component automatically tries to activate
               previosly executed application when this is found
    IDString - string to identify your application, it is highly
               recommended to change it from default random string
             - you really shouldn't change this during run time
    AutoCreate - if true component automatically tries to create Mutex
                 when loaded
    AutoFind - if true component automatically tries to find another
               copy of your application when loaded
             -  AutoCreate must be true, to perform find on loading
    FindOnce - if true components finds another copy of application
               only once (although you run FindID procedure)
    CreateMutexAlso - create also mutex with same name as IDString
  read only:
    Found - true if another copy of application found
    Created - true if message was registered for this copy of application
    Searched - true if another copy of application was searched
    MutexCreateError - true if error pccured while creating mutex - mutex
                       allready exists

EVENTS:
  OnFound - called when another copy of application found
  OnActivate - called when this copy is being activated by another
               copy
  OnMutextCreateError - called when can not create mutex

PROCEDURES:
  CreateID - creates or reads message number, when AutoCreate is True
             you don't have to use this
  FindID - finds another copy of application, FindOnce must be False
           if you want to search again
  ActivateAnother - activates another executed copy of application
}


unit RunOnce;

interface

uses
  Windows,
  SysUtils,
  Classes,
  Controls,
  Forms,
  Messages,
  Dialogs;

type
  TRunOnce = class(TComponent)
  private
    FOnFound: TNotifyEvent;
    FOnActivate: TNotifyEvent;
    FOnMutexErr: TNotifyEvent;

    FIDString: string;

    FTerminate: Boolean;
    FActivate: Boolean;
    FAutoCreate: Boolean;
    FAutoFind: Boolean;
    FFindOnce: Boolean;

    FCreateMutex: Boolean;
    FMutexErr: Boolean;

    FFound: Boolean;
    FCreated: Boolean;
    FSearched: Boolean;

    FHandle: HWND;
    FMutex: HWND;

    WM_RunOnce: UINT;

    procedure WndProc(var oMessage: TMessage);

  public
    procedure Loaded; override;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure CreateID;
    procedure FindID;

    procedure ActivateAnother;

    procedure SendMsg(Msg: Longword);
    procedure PostMsg(Msg: Longword);

  published
    property Found: Boolean read FFound;
    property Created: Boolean read FCreated;
    property Searched: Boolean read FSearched;
    property MutexCreateError: Boolean read FMutexErr;

    property IDString: string read FIDString write FIDString;
    property Terminate: Boolean read FTerminate write FTerminate default True;
    property Activate: Boolean read FActivate write FActivate default True;
    property AutoCreate: Boolean read FAutoCreate write FAutoCreate default True;
    property AutoFind: Boolean read FAutoFind write FAutoFind default True;
    property FindOnce: Boolean read FFindOnce write FFindOnce default False;
    property CreateMutexAlso: Boolean read FCreateMutex write FCreateMutex default True;

    property OnMutextCreateError: TNotifyEvent read FOnMutexErr write FOnMutexErr;
    property OnFound: TNotifyEvent read FOnFound write FOnFound;
    property OnActivate: TNotifyEvent read FOnActivate write FOnActivate;
  end;

procedure Register;

implementation

procedure TRunOnce.Loaded;
begin
  inherited Loaded;
  FCreated := False;
  FFound := False;
  FSearched := False;
  FMutexErr := False;

  if not (csDesigning in ComponentState) then
  begin
    if FAutoCreate then
    begin
      CreateID;
      if FAutoFind then FindID;
    end;
  end;
end;

procedure TRunOnce.CreateID;
begin
  if FCreateMutex then
  begin
    FMutex := CreateMutex(nil, False, PChar(FIDString));
    if (GetLastError = ERROR_ALREADY_EXISTS) or
      (FMutex = 0) then
    begin
      FMutexErr := True;
      if Assigned(FOnMutexErr) then FOnMutexErr(Self);
    end;
  end;

  WM_RunOnce := RegisterWindowMessage(PChar(FIDString));
  if WM_RunOnce <> 0 then
    FCreated := True
  else
    FCreated := False;
end;

procedure TRunOnce.ActivateAnother;
begin
  if FCreated then
    SendMessage(HWND_BROADCAST, WM_RunOnce, 2, Application.Handle);
end;

procedure TRunOnce.WndProc;
begin
  if (oMessage.Msg = WM_RunOnce) and (oMessage.lParam <> Integer(Application.Handle)) then
    case oMessage.wParam of
      0:
        begin
          PostMessage(HWND_BROADCAST, WM_RunOnce, 1, Application.Handle);
          oMessage.Result := 1;
        end;
      1:
        begin
          FFound := True;
          if Assigned(FOnFound) then FOnFound(Self);
          if FTerminate then Application.Terminate;
          if FActivate then ActivateAnother;
          oMessage.Result := 1;
        end;
      2:
        begin
          Application.BringToFront;
          if Assigned(FOnActivate) then FOnActivate(Self);
          oMessage.Result := 1;
        end;
    end
  else
    oMessage.Result := DefWindowProc(FHandle, oMessage.Msg, oMessage.wParam, oMessage.lParam);
end;

procedure TRunOnce.SendMsg(Msg: Longword);
begin
end;

procedure TRunOnce.PostMsg(Msg: Longword);
begin
end;

procedure TRunOnce.FindID;
begin
  if FCreated then
    if not ((FSearched) and (FFindOnce)) then
    begin
      FSearched := True;
      FFound := False;
      PostMessage(HWND_BROADCAST, WM_RunOnce, 0, Application.Handle);
    end;
end;

constructor TRunOnce.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Randomize;

  if not (csDesigning in ComponentState) then FHandle := AllocateHWND(WndProc);

  Str(Round(Random(2147483647)), FIDString);
  FIDString := 'RunOnce' + FIDString;
  FTerminate := True;
  FActivate := True;
  FAutoCreate := True;
  FAutoFind := True;
  FFindOnce := False;
  FCreateMutex := True;
  FOnFound := nil;
  FOnActivate := nil;
  FOnMutexErr := nil;
end;

destructor TRunOnce.Destroy;
begin
  if not (csDesigning in ComponentState) then DeallocateHWND(FHandle);
  inherited Destroy;
end;

procedure Register;
begin
  RegisterComponents('CICOM', [TRunOnce]);
end;

end.

